#ifndef _ARM_CONTROL_H
#define _ARM_CONTROL_H

#include "dynamixelserver_ftdi.h"
#include "dynamixel_motor.h"
#include "threadserver.h"
#include "eventserver.h"
#include "joystick.h"

typedef enum {PAN=0,TILT=1,ROLL=2} motor_id_t;

typedef struct
{
  double pan;
  double tilt;
  double roll;
}TJoints;

class CArmControl
{
  private:
    bool exit_flag;
    CThreadServer *thread_server;
    std::string position_monitor_thread_id;
    CEventServer *event_server;
    std::string finish_thread_event_id;
    std::vector<CDynamixelMotor *> motors;
    CDynamixelServerFTDI *dyn_server;
    CJoystick joy;
    std::vector<bool> current_button_state;
    double max_speed;
    TJoints max_angles;
    TJoints min_angles;
    TJoints max_torques;
    TJoints target_torques;
    std::vector<TJoints> stored_poses;
    std::string poses_file;
  protected:
    static void *position_monitor_thread(void *param);
  public:
    CArmControl(const std::string &joy_device,std::string &dyn_serial, unsigned int baudrate,unsigned char pan_id, unsigned char tilt_id, unsigned char roll_id);
    bool get_button_state(unsigned int button_id);
    bool exit(void);
    // servo functionis
    void set_max_angles(unsigned int axis_id,double max,double min);
    void set_max_torque(unsigned int axis_id,double max);
    void enable_torque(bool enable);
    void move(unsigned int axis_id,double torque);
    void stop(void);
    // stored poses
    void increase_max_speed(void);
    void decrease_max_speed(void);
    void load_poses(const std::string &filename);
    void save_poses(const std::string &filename);
    void update_pose(unsigned int pose_id);
    void execute_pose(unsigned int pose_id);
    ~CArmControl();
};

#endif
